<?php

/*
 * This file is part of the `liip/LiipImagineBundle` project.
 *
 * (c) https://github.com/liip/LiipImagineBundle/graphs/contributors
 *
 * For the full copyright and license information, please view the LICENSE.md
 * file that was distributed with this source code.
 */

namespace Liip\ImagineBundle\Tests\Form\Type;

use Liip\ImagineBundle\Form\Type\ImageType;
use Liip\ImagineBundle\Utility\Framework\SymfonyFramework;
use Liip\ImagineBundle\Tests\AbstractTest;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;

/**
 * @covers \Liip\ImagineBundle\Form\Type\ImageType
 */
class ImageTypeTest extends AbstractTest
{
    public function testGetName()
    {
        $type = new ImageType();

        $this->assertEquals('liip_imagine_image', $type->getName());
    }

    public function testGetParent()
    {
        $type = new ImageType();

        $this->assertEquals('file', $type->getParent());
    }

    public function testConfigureOptions()
    {
        if (SymfonyFramework::isKernelLessThan(2, 6)) {
            $this->markTestSkipped('No need to test on symfony < 2.6');
        }

        $resolver = new OptionsResolver();
        $type = new ImageType();

        $type->configureOptions($resolver);

        $this->assertTrue($resolver->isRequired('image_path'));
        $this->assertTrue($resolver->isRequired('image_filter'));

        $this->assertTrue($resolver->isDefined('image_attr'));
        $this->assertTrue($resolver->isDefined('link_url'));
        $this->assertTrue($resolver->isDefined('link_filter'));
        $this->assertTrue($resolver->isDefined('link_attr'));
    }

    /**
     * @group legacy
     */
    public function testSetDefaultOptionsLegacy()
    {
        if (SymfonyFramework::isKernelGreaterThanOrEqualTo(2, 6)) {
            $this->markTestSkipped('No need to test on symfony >= 2.6');
        }

        $resolver = new OptionsResolver();
        $type = new ImageType();

        $type->setDefaultOptions($resolver);

        $this->assertTrue($resolver->isRequired('image_path'));
        $this->assertTrue($resolver->isRequired('image_filter'));

        $this->assertTrue($resolver->isKnown('image_attr'));
        $this->assertTrue($resolver->isKnown('link_url'));
        $this->assertTrue($resolver->isKnown('link_filter'));
        $this->assertTrue($resolver->isKnown('link_attr'));
    }

    public function testBuildView()
    {
        $options = array(
            'image_path' => 'foo',
            'image_filter' => 'bar',
            'image_attr' => 'bazz',
            'link_url' => 'http://liip.com',
            'link_filter' => 'foo',
            'link_attr' => 'bazz',
        );

        $view = new FormView();
        $type = new ImageType();
        $form = $this->createObjectMock('\Symfony\Component\Form\Test\FormInterface');

        $type->buildView($view, $form, $options);

        foreach ($options as $name => $value) {
            $this->assertArrayHasKey($name, $view->vars);
            $this->assertEquals($value, $view->vars[$name]);
        }
    }
}
